@@ -34,9 +34,9 @@
 #include <QApplication>
 
 #include <QDBusConnection>
+#include <QDBusConnectionInterface>
 #include <QDBusInterface>
 #include <QDBusServiceWatcher>
-#include <QDBusPendingCall>
 
 #include <KDBusService>
 #include <kconfiggroup.h>
@@ -47,16 +47,15 @@
 #include <KPluginInfo>
 #include <KPluginMetaData>
 
-Q_DECLARE_LOGGING_CATEGORY(KDED)
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-// logging category for this framework, default: log stuff >= warning
-Q_LOGGING_CATEGORY(KDED, "kf5.kded", QtWarningMsg)
-#else
-Q_LOGGING_CATEGORY(KDED, "kf5.kded")
+#ifdef Q_OS_OSX
+#include <CoreFoundation/CoreFoundation.h>
 #endif
 
-Kded *Kded::_self = 0;
+Q_DECLARE_LOGGING_CATEGORY(KDED)
+
+Q_LOGGING_CATEGORY(KDED, "kf5.kded", QtWarningMsg)
+
+Kded *Kded::_self = nullptr;
 
 static bool delayedCheck;
 static bool bCheckSycoca;
@@ -71,11 +70,11 @@ extern QDBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage &));
 static void runKonfUpdate()
 {
     KToolInvocation::kdeinitExecWait(QStringLiteral(KCONF_UPDATE_EXE),
-            QStringList(), 0, 0, "0" /*no startup notification*/);
+            QStringList(), nullptr, nullptr, "0" /*no startup notification*/);
 }
 
 Kded::Kded()
-    : m_pDirWatch(0)
+    : m_pDirWatch(nullptr)
     , m_pTimer(new QTimer(this))
     , m_needDelayedCheck(false)
 {
@@ -102,7 +101,7 @@ Kded::Kded()
 
 Kded::~Kded()
 {
-    _self = 0;
+    _self = nullptr;
     m_pTimer->stop();
     delete m_pTimer;
     delete m_pDirWatch;
@@ -136,35 +135,28 @@ void Kded::messageFilter(const QDBusMessage &message)
         return;
     }
 
-    if (self()->m_dontLoad.value(obj, 0)) {
+    if (self()->m_dontLoad.value(obj, nullptr)) {
         return;
     }
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
-    // messageFilter runs in a secondary thread since Qt 5.6, so we use invokeMethod
-    // to load the module in the main thread. But we need to block so that the object
-    // can then process the message.
-    QMetaObject::invokeMethod(self(), "loadModule", Qt::BlockingQueuedConnection, Q_ARG(QString, obj), Q_ARG(bool, true));
-#else
     self()->loadModule(obj, true);
-#endif
 }
 
 static int phaseForModule(const KPluginMetaData &module)
 {
-    const QVariant phasev = module.rawData().value("X-KDE-Kded-phase").toVariant();
+    const QVariant phasev = module.rawData().value(QStringLiteral("X-KDE-Kded-phase")).toVariant();
     return phasev.isValid() ? phasev.toInt() : 2;
 }
 
 QVector<KPluginMetaData> Kded::availableModules() const
 {
-    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins("kf5/kded");
+    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kded"));
     QSet<QString> moduleIds;
     foreach (const KPluginMetaData &md, plugins) {
         moduleIds.insert(md.pluginId());
     }
     // also search for old .desktop based kded modules
-    KPluginInfo::List oldStylePlugins = KPluginInfo::fromServices(KServiceTypeTrader::self()->query("KDEDModule"));
+    KPluginInfo::List oldStylePlugins = KPluginInfo::fromServices(KServiceTypeTrader::self()->query(QStringLiteral("KDEDModule")));
     foreach (const KPluginInfo &info, oldStylePlugins) {
         if (moduleIds.contains(info.pluginName())) {
             qCWarning(KDED).nospace() << "kded module " << info.pluginName() << " has already been found using "
@@ -198,7 +190,14 @@ static KPluginMetaData findModule(const QString &id)
 void Kded::initModules()
 {
     m_dontLoad.clear();
-    bool kde_running = !qgetenv("KDE_FULL_SESSION").isEmpty();
+#ifdef Q_OS_OSX
+    // it seems there is no reason to honour KDE_FULL_SESSION on OS X even if it's set for some reason.
+    // That way kdeinit5 is always auto-started if required (and the kded is as good a candidate
+    // for starting this service as any other application).
+    bool kde_running = false;
+#else
+    bool kde_running = !qEnvironmentVariableIsEmpty("KDE_FULL_SESSION");
+#endif
     if (kde_running) {
 #ifndef Q_OS_WIN
         // not the same user like the one running the session (most likely we're run via sudo or something)
@@ -226,6 +225,9 @@ void Kded::initModules()
     foreach (const KPluginMetaData &module, kdedModules) {
         // Should the service load on startup?
         const bool autoload = isModuleAutoloaded(module);
+        if (!platformSupportsModule(module)) {
+            continue;
+        }
 
         // see ksmserver's README for description of the phases
         bool prevent_autoload = false;
@@ -282,7 +284,7 @@ void Kded::loadSecondPhase()
 
 void Kded::noDemandLoad(const QString &obj)
 {
-    m_dontLoad.insert(obj.toLatin1(), this);
+    m_dontLoad.insert(obj, this);
 }
 
 void Kded::setModuleAutoloading(const QString &obj, bool autoload)
@@ -293,7 +295,7 @@ void Kded::setModuleAutoloading(const QString &obj, bool autoload)
     if (!module.isValid()) {
         return;
     }
-    KConfigGroup cg(config, QString("Module-").append(module.pluginId()));
+    KConfigGroup cg(config, QStringLiteral("Module-").append(module.pluginId()));
     cg.writeEntry("autoload", autoload);
     cg.sync();
 }
@@ -310,11 +312,18 @@ bool Kded::isModuleAutoloaded(const KPluginMetaData &module) const
     }
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
     bool autoload = module.rawData().value(QStringLiteral("X-KDE-Kded-autoload")).toVariant().toBool();
-    KConfigGroup cg(config, QString("Module-").append(module.pluginId()));
+    KConfigGroup cg(config, QStringLiteral("Module-").append(module.pluginId()));
     autoload = cg.readEntry("autoload", autoload);
     return autoload;
 }
 
+bool Kded::platformSupportsModule(const KPluginMetaData &module) const
+{
+    const QStringList supportedPlatforms = KPluginMetaData::readStringList(module.rawData(), QStringLiteral("X-KDE-OnlyShowOnQtPlatforms"));
+
+    return supportedPlatforms.isEmpty() || supportedPlatforms.contains(qApp->platformName());
+}
+
 bool Kded::isModuleLoadedOnDemand(const QString &obj) const
 {
     return isModuleLoadedOnDemand(findModule(obj));
@@ -338,11 +347,11 @@ bool Kded::isModuleLoadedOnDemand(const KPluginMetaData &module) const
 KDEDModule *Kded::loadModule(const QString &obj, bool onDemand)
 {
     // Make sure this method is only called with valid module names.
-    if (obj.contains('/')) {
+    if (obj.contains(QLatin1Char('/'))) {
         qCWarning(KDED) << "attempting to load invalid kded module name:" << obj;
         return nullptr;
     }
-    KDEDModule *module = m_modules.value(obj, 0);
+    KDEDModule *module = m_modules.value(obj, nullptr);
     if (module) {
         return module;
     }
@@ -356,7 +365,7 @@ KDEDModule *Kded::loadModule(const KPluginMetaData &module, bool onDemand)
         return nullptr;
     }
     const QString moduleId = module.pluginId();
-    KDEDModule *oldModule = m_modules.value(moduleId, 0);
+    KDEDModule *oldModule = m_modules.value(moduleId, nullptr);
     if (oldModule) {
         qCDebug(KDED) << "kded module" << moduleId << "is already loaded.";
         return oldModule;
@@ -379,7 +388,7 @@ KDEDModule *Kded::loadModule(const KPluginMetaData &module, bool onDemand)
         kdedModule = factory->create<KDEDModule>(this);
     } else {
         // TODO: remove this fallback code, the kded modules should all be fixed instead
-        KPluginLoader loader2("kded_" + module.fileName());
+        KPluginLoader loader2(QStringLiteral("kded_") + module.fileName());
         factory = loader2.factory();
         if (factory) {
             qCWarning(KDED).nospace() << "found kded module " << moduleId
@@ -404,7 +413,7 @@ KDEDModule *Kded::loadModule(const KPluginMetaData &module, bool onDemand)
 
 bool Kded::unloadModule(const QString &obj)
 {
-    KDEDModule *module = m_modules.value(obj, 0);
+    KDEDModule *module = m_modules.value(obj, nullptr);
     if (!module) {
         return false;
     }
@@ -524,7 +533,7 @@ void Kded::recreate(bool initial)
         recreateDone();
         if (delayedCheck) {
             // do a proper ksycoca check after a delay
-            QTimer::singleShot(60000, this, SLOT(runDelayedCheck()));
+            QTimer::singleShot(60000, this, &Kded::runDelayedCheck);
             m_needDelayedCheck = true;
             delayedCheck = false;
         } else {
@@ -618,14 +627,14 @@ KUpdateD::KUpdateD()
     QObject::connect(m_pDirWatch, &KDirWatch::dirty,
                      this, &KUpdateD::slotNewUpdateFile);
 
-    const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "kconf_update", QStandardPaths::LocateDirectory);
+    const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kconf_update"), QStandardPaths::LocateDirectory);
     for (QStringList::ConstIterator it = dirs.begin();
             it != dirs.end();
             ++it) {
         QString path = *it;
         Q_ASSERT(path != QDir::homePath());
-        if (path[path.length() - 1] != '/') {
-            path += '/';
+        if (path[path.length() - 1] != QLatin1Char('/')) {
+            path += QLatin1Char('/');
         }
 
         if (!m_pDirWatch->contains(path)) {
@@ -674,14 +683,28 @@ void KBuildsycocaAdaptor::enableTestMode()
 
 static void setupAppInfo(QApplication *app)
 {
-    app->setApplicationName("kded5");
-    app->setApplicationDisplayName("KDE Daemon");
-    app->setOrganizationDomain("kde.org");
-    app->setApplicationVersion(KDED_VERSION_STRING);
+    app->setApplicationName(QStringLiteral("kded5"));
+    app->setApplicationDisplayName(QStringLiteral("KDE Daemon"));
+    app->setOrganizationDomain(QStringLiteral("kde.org"));
+    app->setApplicationVersion(QStringLiteral(KDED_VERSION_STRING));
 }
 
 extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
 {
+#ifdef Q_OS_OSX
+    CFBundleRef mainBundle = CFBundleGetMainBundle();
+    if (mainBundle) {
+        // get the application's Info Dictionary. For app bundles this would live in the bundle's Info.plist,
+        // for regular executables it is obtained in another way.
+        CFMutableDictionaryRef infoDict = (CFMutableDictionaryRef) CFBundleGetInfoDictionary(mainBundle);
+        if (infoDict) {
+            // Add or set the "LSUIElement" key with/to value "1". This can simply be a CFString.
+            CFDictionarySetValue(infoDict, CFSTR("LSUIElement"), CFSTR("1"));
+            // That's it. We're now considered as an "agent" by the window server, and thus will have
+            // neither menubar nor presence in the Dock or App Switcher.
+        }
+    }
+#endif
     //options.add("check", qi18n("Check Sycoca database only once"));
 
     // WABA: Make sure not to enable session management.
@@ -711,6 +734,21 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
 
     KDBusService service(KDBusService::Unique);
 
+    QDBusConnectionInterface *bus = QDBusConnection::sessionBus().interface();
+    // Also register as all the names we should respond to (org.kde.kcookiejar, org.kde.khotkeys etc.)
+    // so that the calling code is independent from the physical "location" of the service.
+    const QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kded"));
+    foreach (const KPluginMetaData &metaData, plugins) {
+        const QString serviceName = metaData.rawData().value(QStringLiteral("X-KDE-DBus-ServiceName")).toString();
+        if (serviceName.isEmpty()) {
+            qCWarning(KDED) << "No X-KDE-DBus-ServiceName found in" << metaData.fileName();
+            continue;
+        }
+        if (!bus->registerService(serviceName)) {
+            qCWarning(KDED) << "Couldn't register name" << serviceName << "with DBUS - another process owns it already!";
+        }
+    }
+
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
     KConfigGroup cg(config, "General");
 
@@ -733,28 +771,8 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
         (void) new KUpdateD;    // Watch for updates
     }
 
-//NOTE: We are going to change how KDE starts and this certanly doesn't fit on the new design.
-#ifdef Q_OS_LINUX
-    // Tell KSplash that KDED has started
-    QDBusMessage ksplashProgressMessage = QDBusMessage::createMethodCall(QStringLiteral("org.kde.KSplash"),
-                                          QStringLiteral("/KSplash"),
-                                          QStringLiteral("org.kde.KSplash"),
-                                          QStringLiteral("setStage"));
-    ksplashProgressMessage.setArguments(QList<QVariant>() << QStringLiteral("kded"));
-    QDBusConnection::sessionBus().asyncCall(ksplashProgressMessage);
-#endif
-
     runKonfUpdate(); // Run it once.
 
-#ifdef Q_OS_LINUX
-    ksplashProgressMessage = QDBusMessage::createMethodCall(QStringLiteral("org.kde.KSplash"),
-                             QStringLiteral("/KSplash"),
-                             QStringLiteral("org.kde.KSplash"),
-                             QStringLiteral("setStage"));
-    ksplashProgressMessage.setArguments(QList<QVariant>() << QStringLiteral("confupdate"));
-    QDBusConnection::sessionBus().asyncCall(ksplashProgressMessage);
-#endif
-
     int result = app.exec(); // keep running
 
     delete kded;
